from pathlib import Path
from typing import List

import tox

_REQUIREMENTS = Path("requirements")

_HEADER = """\
#
# This file is autogenerated by tox-constraints
# It is updated every time tox runs
#
"""


def _patch_envconfigs(envconfigs):
    for envconfig in envconfigs.values():
        envconfig.install_command.append("-cconstraints.txt")
        if envconfig.skip_install is True:
            pass
        elif envconfig.skip_install is False:
            envconfig.install_command.append(
                "-r" + str(_REQUIREMENTS / "install_requires.txt")
            )
        else:
            raise ValueError


def _export_deps(envconfigs):
    deps: List[str] = []
    for name, envconfig in envconfigs.items():
        filename = f"tox-{name}.txt"
        filepath = _REQUIREMENTS / filename
        with filepath.open("w") as fp:
            for dep in envconfig.deps:
                fp.write(_HEADER)
                fp.write(f"{dep}\n")
        deps.append("-r" + str(filename))

    with (_REQUIREMENTS / "tox.txt").open("w") as fp:
        fp.write(_HEADER)
        for dep in deps:
            fp.write(f"{dep}\n")


@tox.hookimpl
def tox_configure(config):
    _patch_envconfigs(config.envconfigs)
    _export_deps(config.envconfigs)

    return None
